import logging

from paircd.client import Client
from paircd.command_handler import CommandHandler
from paircd.message import Message
from paircd.server import Server


class JoinHandler(CommandHandler):
    def __init__(self) -> None:
        super().__init__("JOIN", 1)

    async def handle(self, server: Server, client: Client, msg: Message) -> None:
        if not client.registered:
            client.log("join: not registered", level=logging.WARN)
            return

        channel_name = msg.args[0]
        if not channel_name.startswith("#"):
            client.log("tried to join invalid channel", level=logging.WARN)
            return

        channel = server.get_channel_by_name(channel_name)
        channel.add_client(client)

        client.channels.add(channel_name)
        client.log(f"joined {channel_name}")

        await channel.msg_queue.put(
            Message(cmd="JOIN", args=[channel_name], prefix=client.id())
        )
